//+------------------------------------------------------------------+
//|                                                   MeraPengar.mq4 |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright ""
#property link      ""

/**************
Jan 20, 2007, 11:52 AM; sdln28

I have this great Indicator and I would like it to be an EA.

Currently when the lines cross you open a trade in the direction of the movement.
Works best for GBP/USD and USD/JPY - M30 timeframe. It is currently an indicator for
manual trading but if someone could help I was hoping on the cross the EA could open
a pending order (buystop or sellstop) in the direction of the move 12 pips out from
the cross. The instead of the audio alert maybe it could send an e-mail. The EA will
probably only trade once or twice a month but the profits will be big (usually 200
pips before the next cross) - currently I am doing great with it manually if I see
the cross - that is why I want it to be an EA instead of an indicator - so it will
trade itself. Lot size will probably need to be an input so this will work for what
fits the user's needs.
***************/

#include <stderror.mqh>

//---- input parameters
extern double lots = 1.0; // Volume to open trades with
extern int delta = 12; // #pips for trade stop/limit
extern int delay = 4; // #ignored bars after a crossing
extern int margin = 75; // margin call percentage

string comment = "Yet again";
int magic = 42;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
    delta = MathMax( delta, MarketInfo( Symbol(), MODE_STOPLEVEL ) );
    return(0);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
    return(0);
}
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
    int cmd;

    // Hold back on cross detection close to a newly detected cross
    static datetime last_cross_time = 0;
    if ( TimeCurrent() <= last_cross_time + delay * Period() ) {
        return( 0 );
    }

    // Check for a cross
    if ( ! cross( cmd ) ) {
        return( 0 );
    }

    last_cross_time = TimeCurrent(); // Remember when the cross was detected

    Print( "cross " + last_cross_time );
    
    // Close any opposite trades (with right magic)
    closeTrades( cmd );
    
    // Check for tradable margin
    if ( AccountFreeMargin() < AccountEquity() * margin / 100.0 ) {
        // Not enough margin for a trade
        Print( "not enough margin" );
        return( 0 );
    }

    // Open the indicated pending position
    trade( cmd );
    return(0);
}

//+------------------------------------------------------------------+
//| indicator function, replicating the code of CrossAlertsTrader    |
//| returns true when a cross is detected, and then its given        |
//| variable is set as OP_BUY or OP_SELL to tell final trend         |
//+------------------------------------------------------------------+
bool cross(int &to)
{
    static int direction = 0; // No direction to start with.
    bool crossed = false;
    to = -1;

    if ( iMA( NULL, 0, 144, 0, MODE_EMA, PRICE_CLOSE, 0 ) >
         iMA( NULL, 0, 200, 0, MODE_EMA, PRICE_CLOSE, 0 ) ) {
         // blue line > red line
         crossed = direction < 0 ;
         to = OP_BUYSTOP;
         direction = 1;
         Comment( "up" );
    } else {
         // blue line < red line
         crossed = direction > 0;
         direction = -1;
         to = OP_SELLSTOP;
         Comment( "down" );
    }
    return ( crossed );
}

//+------------------------------------------------------------------+
//| trading function, open a position in the given direction using   |
//| global lots, slippage, stoploss, takeprofit, comment and magic   |
//| it also invokes notify() for trading notification                |
//+------------------------------------------------------------------+
void trade(int cmd)
{
    Print( "trade " + cmd );

    int ticket = -1;
    double price = 0;
    int slippage = 1;
    double sl = 0;
    double tp = 0;
    
    for ( ;; ) {
        if ( cmd == OP_BUYSTOP ) {
            price = NormalizeDouble( Ask + delta * Point, Digits );
        } else if ( cmd == OP_SELLSTOP ) {
            price = NormalizeDouble( Bid - delta * Point, Digits );
        } else {
            break; // Ignore other cmd
        }
        ticket = OrderSend( Symbol(), cmd, lots, price, slippage, sl, tp, comment, magic );
        if ( ticket > 0 ) {
            notify( ticket );
            break;
        }
        int error = GetLastError();
        Print( "trade error " + error );
        if ( retryError( error ) ) {
            Sleep( 5000 );
            RefreshRates();
        }
        break;
    }
}

//+------------------------------------------------------------------+
//| closing trades function, closing all opened trades of right magic|
//| and of this symbol                                               |
//+------------------------------------------------------------------+
void closeTrades(int cmd)
{
    double price = 0;
    int slippage = 1;
    for ( int i = OrdersTotal(); i >= 0; i-- ) {
        if ( ! OrderSelect( i, SELECT_BY_POS ) )
            continue;
        if ( OrderMagicNumber() != magic )
            continue;
        if ( OrderSymbol() != Symbol() )
            continue;
        if ( cmd == OP_BUYSTOP ) {
            if ( OrderType() != OP_SELL )
                continue;
            price = Ask;
        } else if ( cmd == OP_SELLSTOP ) {
            if ( OrderType() != OP_BUY )
                continue;
            price = Bid;
        }
        if ( OrderClose( OrderTicket(), OrderLots(), price, slippage ) )
            continue;
        int error = GetLastError();
        Print( "Close error " + error );
        if ( retryError( error ) ) {
            Sleep( 2000 );
            RefreshRates();
            i++; // Retrying the index
        }
    }
}

//+------------------------------------------------------------------+
//| notification function, to notify about an opened trade           |
//+------------------------------------------------------------------+
void notify(int ticket)
{
      Alert("EMA Cross for "+Symbol()+" on the "+Period()+" minute chart.");
}

//+------------------------------------------------------------------+
//| recognise all OrderSend errors for which the trade action should |
//| be tried again in a moment.                                      |
//+------------------------------------------------------------------+
int retry_errors[] = {
    ERR_SERVER_BUSY,
    ERR_NO_CONNECTION,
    ERR_TOO_FREQUENT_REQUESTS,
    ERR_TRADE_TIMEOUT,
    ERR_PRICE_CHANGED,
    ERR_OFF_QUOTES,
    ERR_REQUOTE,
    ERR_TOO_MANY_REQUESTS,
    142, 143, 144,
    ERR_TRADE_CONTEXT_BUSY
};

bool retryError(int error) {
    for ( int i = 0; i < ArraySize( retry_errors ); i++ ) {
        if ( retry_errors[ i ] == error )
            return( true );
    }
    return( false );
}

//+------------------------------------------------------------------+